# -*- coding: utf-8 -*-

import numpy as np
import skimage.io as skimage_io
from skimage.draw import polygon
from io import BytesIO
from PIL import Image


def coordstringtoarray(coordstring, scale=1):
    coords = [p.split(",") for p in coordstring.split()]
    return np.array([(int(scale*int(c[1])), int(scale*int(c[0])))
                     for c in coords])


def expandcoords(coords, imgshape, context, sides=1, rcoords=None):
    if type(coords) != np.ndarray:
        coords = coordstringtoarray(coords)
    if rcoords and type(rcoords) != np.ndarray:
        rcoords = coordstringtoarray(rcoords)
    lineh = max([p[0] for p in coords]) - min([p[0] for p in coords])
    xmin = max(0, int(min(p[0] for p in coords) - (context * lineh)))
    xmax = min(imgshape[1], int(max(p[0] for p in coords) + (context * lineh)))
    ymin = min(p[1] for p in rcoords)
    ymax = max(p[1] for p in rcoords)
    return np.array([(xmin, ymin), (xmin, ymax), (xmax, ymax), (xmax, ymin)])


def cutout(pageimg, coords, scale=1):
    """Cuts out coords from pageimg."""
    if len(pageimg.shape) > 2:
        pageimg = pageimg[:, :, 0]
    if type(coords) != np.ndarray:
        coords = coordstringtoarray(coords, scale)
    rr, cc = polygon(coords[:, 0], coords[:, 1], pageimg.shape)
    offset = (min([x[0] for x in coords]), min([x[1] for x in coords]))
    box = np.ones(
        (max([x[0] for x in coords]) - offset[0],
         max([x[1] for x in coords]) - offset[1]),
        dtype=pageimg.dtype) * 255
    box[rr-offset[0], cc-offset[1]] = pageimg[rr, cc]
    return box, offset


def getsnippet(filename, coords, imgshape=None, context=0, rcoords=None):
    pageimg = skimage_io.imread(filename)
    scale = 1
    if imgshape:
        scale = pageimg.shape[1] / imgshape[0]
    if context:
        coords = expandcoords(coords, imgshape, context, rcoords=rcoords)
    cut = cutout(pageimg, coords, scale)[0]
    file = BytesIO()
    image = Image.fromarray(cut)
    image.save(file, 'png')
    file.name = "line.png"
    file.seek(0)
    return file.read()



# -*- coding: utf-8 -*-

import zipfile

from nashi.models import Book, Page
from nashi.database import db_session

from glob import glob
from os import path, mkdir, symlink, chmod
from shutil import chown, copy
from lxml import etree
from sqlalchemy.orm.exc import NoResultFound


def scan_bookfolder(bookfolder, img_subdir):
    """ Scan bookfolder and write book info to database. """
    books = glob(bookfolder + "//*/")
    for bookpath in books:
        bookname = path.split(bookpath[:-1])[1]
        files = set([f.split(sep=".")[0] for f in glob(bookpath + img_subdir
                                                       + "*.png")])
        no_pages_total = len(files)
        book = Book.query.filter_by(name=bookname).first()
        if not book:
            book = Book(name=bookname, no_pages_total=no_pages_total)
            db_session.add(book)
        else:
            book.no_pages_total = no_pages_total
    db_session.commit()


def upload_pagexml(file):
    try:
        zf = zipfile.ZipFile(file._file)
    except zipfile.BadZipFile:
        return "Upload failed. Please upload a valid zip file."
    result = {}
    for fn in zf.namelist():
        try:
            bookname, filename = fn.split("/")
        except ValueError:
            return "Upload failed. The files inside the zip file have to be" +\
                   " named <BOOKNAME>/<PAGENAME>.xml."
        if not filename:
            continue
        try:
            book = Book.query.filter_by(name=bookname).one()
        except NoResultFound:
            return "Import aborted. Book {} is not in your library."\
                    .format(bookname)
        if bookname not in result:
            result[bookname] = 0
        pagename = path.splitext(path.split(filename)[1])[0]
        try:
            page = Page.query.filter_by(book_id=book.id, name=pagename).one()
        except NoResultFound:
            page = Page(book=book, name=pagename)
        # return "Import aborted. Book {}, page {} is not in your library."\
        #       .format(bookname, pagename)
        pagexml = zf.read(fn).decode("utf-8")
        root = etree.fromstring(pagexml)
        ns = {"ns": root.nsmap[None]}
        page.no_lines_segm = int(root.xpath("count(//ns:TextLine)",
                                            namespaces=ns))
        page.no_lines_gt = int(root.xpath(
            'count(//ns:TextLine/ns:TextEquiv[@index="0"])', namespaces=ns))
        page.no_lines_ocr = int(root.xpath('count(//ns:TextLine'
                                           '[count(./ns:TextEquiv'
                                           '[@index>0])>0])', namespaces=ns))
        page.data = etree.tounicode(root.getroottree())
        result[bookname] += 1
    db_session.commit()
    res = "Import successfull: {}.".format(", ".join(
        [": ".join([i[0], str(i[1])]) for i in result.items()]))
    return res


def getlayers(book):
    layers = set([])
    for p in Page.query.filter_by(book_id=book.id):
        root = etree.fromstring(p.data)
        ns = {"ns": root.nsmap[None]}
        nl = set(root.xpath('//ns:TextEquiv/@index', namespaces=ns))
        layers = layers | nl
    return sorted(list(layers), key=int)


def copy_to_larex(bookname, booksdir, img_subdir, larexdir, larexgrp):
    pngfiles = glob("{}/{}/{}/*.png".format(booksdir, bookname, img_subdir))
    pngfiles = [path.abspath(f) for f in pngfiles]
    pages = set([path.split(f)[1].split(".")[0] for f in pngfiles])
    cpimgs = []
    for p in sorted(pages):
        fullbinp = path.abspath("{}/{}/{}/{}.bin.png"
                                .format(booksdir, bookname, img_subdir, p))
        fullnrmp = path.abspath("{}/{}/{}/{}.nrm.png"
                                .format(booksdir, bookname, img_subdir, p))
        if fullbinp in pngfiles:
            cpimgs.append(fullbinp)
        elif fullnrmp in pngfiles:
            cpimgs.append(fullnrmp)
        else:
            cpimgs.append([x for x in pngfiles if path.split(x)[1].startswith(
                p+".")][0])
    if cpimgs:
        if not path.isdir(larexdir + "/" + bookname):
            mkdir(larexdir + "/" + bookname)
            chmod(larexdir + "/" + bookname, 0o770)
            try:
                chown(larexdir + "/" + bookname, group=larexgrp)
            except PermissionError:
                pass  # Needs error handling, maybe flash?
        for f in cpimgs:
            fname = path.split(f)[1]
            if not path.isfile("{}/{}/{}".format(larexdir, bookname, fname)):
                symlink(f, "{}/{}/{}".format(larexdir, bookname, fname))
            # copy(f, "{}/{}/{}".format(larexdir, bookname, fname))
    return len(cpimgs)



from setuptools import setup

with open("./README.md") as f:
    readme = f.read()

setup(
    name='nashi',
    version='0.0.37',
    license='GPL_v3.0',
    author='Andreas Büttner',
    author_email='andreas.buettner@uni-wuerzburg.de',
    description='A webapp for the transcription of scanned pages',
    url='https://github.com/andbue/nashi',
    long_description=readme,
    long_description_content_type='text/markdown',
    packages=['nashi'],
    include_package_data=True,
    zip_safe=False,
    entry_points={
        'console_scripts': [
            'nashi-import=nashi.import_from_pagexml:main',
            'nashi-export=nashi.import_from_pagexml:bookexport',
            'nashi-delete=nashi.import_from_pagexml:bookdelete'
        ],
    },
    python_requires='>=3',
    install_requires=[
        'sqlalchemy',
        'flask_security',
        'bcrypt',
        'flask_bootstrap',
        'flask_nav',
        'celery',
        'redis',
        'lxml',
        'pillow',
        'krakensegment',
        'scikit-image',
        'Flask'
    ],
    classifiers=[
        'Programming Language :: Python :: 3',
        'Programming Language :: JavaScript',
        'Development Status :: 3 - Alpha',
        'Framework :: Flask',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
    ],
)
